/*----------------------------------------------------------------------
  File    : bvnorm.h
  Contents: Bivariate normal distribution management
  Author  : Christian Borgelt
  History : 2000.01.14 file created
            2000.01.31 function bvn_ellphi added
            2000.02.01 functions bvn_ellx2y and bvn_elly2x added
            2004.04.27 function bvn_dist added
----------------------------------------------------------------------*/
#ifndef __BVNORM__
#define __BVNORM__

/*----------------------------------------------------------------------
  Type Definitions
----------------------------------------------------------------------*/
typedef struct {                /* --- 2-dim. normal distribution --- */
  double prob;                  /* prior probability */
  double norm;                  /* normalization factor */
  double det;                   /* determinant of covariance matrix */
  double ex, ey;                /* expected values */
  double dx, dy;                /* standard deviations */
  double cxx, cyy, cxy;         /* covariance matrix */
  double ixx, iyy, ixy;         /* inverse of covariance matrix */
  double dxx, dyy, dxy;         /* Cholesky decomp. of cov. matrix */
} BVNORM;                       /* (2-dim. normal distribution) */

/*----------------------------------------------------------------------
  Functions
----------------------------------------------------------------------*/
extern int    bvn_init   (BVNORM *bvn, double prob,
                          double ex,  double ey,
                          double cxx, double cyy, double cxy);
extern double bvn_prob   (BVNORM *bvn);
extern double bvn_ex     (BVNORM *bvn);
extern double bvn_ey     (BVNORM *bvn);
extern double bvn_dx     (BVNORM *bvn);
extern double bvn_dy     (BVNORM *bvn);
extern double bvn_cxx    (BVNORM *bvn);
extern double bvn_cyy    (BVNORM *bvn);
extern double bvn_cxy    (BVNORM *bvn);
extern double bvn_ixx    (BVNORM *bvn);
extern double bvn_iyy    (BVNORM *bvn);
extern double bvn_ixy    (BVNORM *bvn);
extern double bvn_max    (BVNORM *bvn);
extern double bvn_dist   (BVNORM *bvn, double x, double y);
extern double bvn_eval   (BVNORM *bvn, double x, double y);
extern void   bvn_ellx2y (BVNORM *bvn, double k, double x,
                          double *y1, double *y2);
extern void   bvn_elly2x (BVNORM *bvn, double k, double y,
                          double *x1, double *x2);
extern void   bvn_ellphi (BVNORM *bvn, double k, double phi,
                          double *x, double *y);

/*----------------------------------------------------------------------
  Preprocessor Definitions
----------------------------------------------------------------------*/
#define bvn_prob(d)      ((d)->prob)
#define bvn_ex(d)        ((d)->ex)
#define bvn_ey(d)        ((d)->ey)
#define bvn_dx(d)        ((d)->dx)
#define bvn_dy(d)        ((d)->dy)
#define bvn_cxx(d)       ((d)->cxx)
#define bvn_cyy(d)       ((d)->cyy)
#define bvn_cxy(d)       ((d)->cxy)
#define bvn_ixx(d)       ((d)->ixx)
#define bvn_iyy(d)       ((d)->iyy)
#define bvn_ixy(d)       ((d)->ixy)
#define bvn_max(d)       ((d)->norm)

#endif
